/*------------------------------------------------------------------------------------*/
/*                                                                                    */
/*    Copyright (C) 2008 Georges Qunot, LIG-CNRS.                                    */
/*    Version 1.00 Last revision: December 2, 2008.                                   */
/*                                                                                    */
/*    This file is part of KNNLSB (K Nearest Neighbors Linear Scan Baseline).         */
/*                                                                                    */
/*    KNNLSB is free software: you can redistribute it and/or modify                  */
/*    it under the terms of the GNU General Public License as published by            */
/*    the Free Software Foundation, either version 3 of the License, or               */
/*    (at your option) any later version.                                             */
/*                                                                                    */
/*    KNNLSB is distributed in the hope that it will be useful,                       */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of                  */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                   */
/*    GNU General Public License for more details.                                    */
/*                                                                                    */
/*    You should have received a copy of the GNU General Public License               */
/*    along with KNNLSB.  If not, see <http://www.gnu.org/licenses/>.                 */
/*                                                                                    */
/*------------------------------------------------------------------------------------*/

#define fdata double
#define FDATA_MAX DBL_MAX

/*------------------------------------------------------------------------------------*/

#include <sched.h>
#include <pthread.h>
#include <stdlib.h>
#include <stdio.h>
#include <float.h>
#include <math.h>
#include <string.h>
#include <sys/time.h>

/*------------------------------------------------------------------------------------*/

typedef struct {
  int k;
  fdata d;
} KEY;

typedef struct {
  KEY **key;
  fdata **ma;
  fdata **mb;
  fdata *sa;
  fdata *sb;
  fdata *dm;
  int la;
  int ha;
  int lb;
  int hb;
  int nc;
  int nk;
  int np;
  int dc;
} KNNPARAMS;

size_t UsedMemory();
size_t MaxUsedMemory();
void *malloc1a(int n0, size_t size);
void free1a(void *v1, int n0, size_t size);
void *malloc2a(int n0, int n1, size_t size);
void free2a(void *v2, int n0, int n1, size_t size);

int get_nb_cores();
int get_cache_size();

fdata *getnorm(fdata **m, int n, int nc, int dc);
KEY **knnp(fdata **ma, fdata **mb, fdata *sa, int na, int nb, int nc, int nk,
int np, int dc, int sta);

KEY **knnc(fdata **ma, fdata **mb, int na, int nb, int nc, int nk, int dc, int sta);
void knncheck(KEY **key0, KEY **key1, int nb, int nk);

/*------------------------------------------------------------------------------------*/
