/*------------------------------------------------------------------------------------*/
/*                                                                                    */
/*    Copyright (C) 2008 Georges Qunot, LIG-CNRS.                                    */
/*    Version 1.00 Last revision: December 2, 2008.                                   */
/*                                                                                    */
/*    This file is part of KNNLSB (K Nearest Neighbors Linear Scan Baseline).         */
/*                                                                                    */
/*    KNNLSB is free software: you can redistribute it and/or modify                  */
/*    it under the terms of the GNU General Public License as published by            */
/*    the Free Software Foundation, either version 3 of the License, or               */
/*    (at your option) any later version.                                             */
/*                                                                                    */
/*    KNNLSB is distributed in the hope that it will be useful,                       */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of                  */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                   */
/*    GNU General Public License for more details.                                    */
/*                                                                                    */
/*    You should have received a copy of the GNU General Public License               */
/*    along with KNNLSB.  If not, see <http://www.gnu.org/licenses/>.                 */
/*                                                                                    */
/*------------------------------------------------------------------------------------*/

#include "knnp.h"

#define NC 100
#define NA 100000
#define NK 10
#define NB 1000

/*------------------------------------------------------------------------------------*/

#define FAILURE 0
#define SUCCESS 1

int argindex(int argc, char *argv[], char *name, int n)
{
  int i;
  for (i = 1; i < argc-n; i++) if (!strcmp(name,argv[i])) return(i);
  return(0);
}

void argremove(int *pargc, char *argv[], int i, int n)
{
  int j;
  *pargc -= n+1;
  for (j = i; j < *pargc; j++) argv[j] = argv[j+n+1];
}

int getbarg(int *pargc, char *argv[], char *name)
{
  int i;
  if ((i = argindex(*pargc,argv,name,0))) argremove(pargc,argv,i,0);
  return(i != 0);
}

int getiarg(int *pargc, char *argv[], char *name, int idef, int *piarg)
{
  int i;
  *piarg = idef;
  if ((i = argindex(*pargc,argv,name,1))) {
    if (sscanf(argv[i+1],"%d",piarg) != 1) {
      fprintf(stderr,"ERROR: %s option expects a valid integer\n",name);
      return(FAILURE);
    }
    argremove(pargc,argv,i,1);
  }
  return(SUCCESS);
}

/*------------------------------------------------------------------------------------*/

fdata **genrand(int n, int nc)
{
  int i,ic;
  fdata **m;
  if ((m = malloc2a(n,nc,sizeof(fdata))) == NULL) return(NULL);
  for (i = 0; i < n; i++) for (ic = 0; ic < nc; ic++)
    m[i][ic] = (rand()&0xFFFFFF)/((fdata) 0xFFFFFF);
  return(m);
}

/*------------------------------------------------------------------------------------*/

int main(int argc, char *argv[])
{
  int ia,ib,ic,ik,p,np,na,nb,nc,nk,dc,pc,ch;
  fdata **ma,**mb,*sa,*va;
  KEY **key0,**key1;
  
  if (getiarg(&argc,argv,"-NA",NA,&na) == FAILURE) exit(1);
  if (getiarg(&argc,argv,"-NB",NB,&nb) == FAILURE) exit(1);
  if (getiarg(&argc,argv,"-NC",NC,&nc) == FAILURE) exit(1);
  if (getiarg(&argc,argv,"-NK",NK,&nk) == FAILURE) exit(1);
  if (getiarg(&argc,argv,"-NP",get_nb_cores(),&np) == FAILURE) exit(1);
  dc = (getbarg(&argc,argv,"-COS"));
  pc = (getbarg(&argc,argv,"-PRE"));
  ch = (getbarg(&argc,argv,"-CHK"));

  if ((ma = genrand(na,nc)) == NULL) exit(1);
  if ((mb = genrand(nb,nc)) == NULL) exit(1);

  sa = pc ? getnorm(ma,na,nc,dc) : NULL;
  if (pc && !sa) exit(1);
  
  printf("%d-core, cache size: %d KB\n\n",get_nb_cores(),get_cache_size()/1024);

  for (p = 1; p <= np; p++) {
    if ((key0 = knnp(ma,mb,sa,na,nb,nc,nk,p,dc,1)) == NULL) exit(1);
    if (p < np) free2a(key0,nb,nk,sizeof(KEY));
  }
  for (ib = 0; ib < 1; ib++) {
    printf("\n");
    for (ik = 0; (ik < nk) && (ik < 10); ik++)
      printf("%9d  %f\n",key0[ib][ik].k,key0[ib][ik].d);
    printf("\n");
  }
  
  if (ch) {
    if ((key1 = knnc(ma,mb,na,nb,nc,nk,dc,1)) == NULL) exit(1);
    for (ib = 0; ib < 1; ib++) {
      printf("\n");
      for (ik = 0; (ik < nk) && (ik < 10); ik++)
        printf("%9d  %f\n",key1[ib][ik].k,key1[ib][ik].d);
      printf("\n");
    }
    knncheck(key0,key1,nb,nk);
    free2a(key1,nb,nk,sizeof(KEY));
  }
  
  free2a(ma,na,nc,sizeof(fdata));
  free2a(mb,nb,nc,sizeof(fdata));
  free2a(key0,nb,nk,sizeof(KEY));
  if (pc) free1a(sa,na,sizeof(fdata));
  
  printf("Maximum allocated memory: %12zu bytes\n",MaxUsedMemory());
  printf("Still allocated memory:   %12zu bytes\n",UsedMemory());
  printf("\nDone\n");
  
  exit(0);
}

/*------------------------------------------------------------------------------------*/
